""" framework to program bots. """

__txt__ = "framework to program bots."
__version__ = 2

## imports

import boht.base
import boht.clock
import boht.cmds
import boht.command
import boht.entry
import boht.fleet 
import boht.loader
import boht.handler
import boht.shell
import boht.static
import boht.tasks
import boht.trace
import boht.users
import boht.utils

import logging
import os
import time
import threading

## vars

#:
homedir = os.path.expanduser("~")
#:
starttime = time.time()

def ld(*args):
    """ create filename in the logdir. """
    return os.path.abspath(os.path.join(cfg.logdir, *args))

def hd(*args):
    """ return a subdir in the homedir. """
    return os.path.abspath(os.path.join(homedir, *args))

def wd(*args):
    """ create pathname in the working directory. """
    workdir = os.path.abspath(cfg.workdir)
    return os.path.abspath(os.path.join(workdir, *args))

class Cfg(boht.base.Object):

    """ basic config. """

#:
cfg = Cfg()
cfg.args = []
cfg.exclude = ""
cfg.history = False
cfg.modules = ""
cfg.options = ""
cfg.name = "boht"
cfg.verbose = False
cfg.shell = False
cfg.workdir = hd(".boht")
cfg.logdir = wd("logs")
cfg.storedir = wd("store")
cfg.version = __version__

## classes

class Bot(boht.handler.Handler):

    """ basic bot. """

    def __init__(self, *args, **kwargs):
        super().__init__()
        self._prompted = threading.Event()
        self.cfg = Cfg()
        self.cfg.update(kwargs)
        self.channels = []
        self.state = boht.base.Object()
        self.verbose = True

    def announce(self, txt):
        """ announce txt on all channels. """
        for channel in self.channels:
             self.say(channel, txt)

    def cmd(self, txt, options="", origin=""):
        """ exec a command. """
        self.start()
        e = Event()
        e.txt = txt
        e.orig = repr(self)
        e.options = options
        e.origin = origin or "root@shell"
        self.dispatch(e)
        self.ready()
        return e

    def dispatch(self, event):
        """ cast to boht.Event """
        e = Event()
        e.update(event)
        return super().dispatch(e)

    def join(self):
        """ bot's could be joined. """
        pass

    def raw(self, txt):
        """ use print() to display text. """
        print(txt)

    def resume(self):
        """ virtual resume. """
        pass

    def say(self, id, channel, txt):
        """ take text part and display it. """
        if self.verbose:
            self.raw(txt)

    def start(self):
        """ add this bot to the kernel. """
        super().start()
        boht.run.kernel.add(self)

class Event(boht.command.Command):

    """ basic event. """

    def ok(self, txt):
        """ send ok reply. """
        self.reply("ok %s" % txt)

    def prompt(self):
        """ prompt virtual member. """
        pass

    def reply(self, txt):
        """ add text to the result. """
        self._result.append(txt)

    def say(self, orig, channel, txt, type=None):
        """ use print if self.verbose. """
        if self.verbose:
            print(txt)

    def show(self):
        """ broadcast result to bus. """
        if not self._result:
            return
        for txt in self._result:
            boht.run.kernel.say(self.orig, self.channel, txt)
        b = boht.run.kernel.get_bot(self.orig)
        if b and "_prompted" in dir(b):
            b._prompted.set()

## functions

def cmd(txt):
    """ run a single command. """
    e = boht.run.kernel.cmd(txt)
    e.wait()
    return e

def edit(o, setter):
    """ edit a object. """
    for key, value in setter.items():
        if "," in value:
            value = value.split(",")
        t = type(value)
        if value in ["True", "true"]:
            o.set(key, True)
        elif value in ["False", "false"]:
            o.set(key, False)
        elif t == list:
            o.set(key, value)
        elif t == str:
            o.set(key, value)
        else:
            o.set(key, value)
    return o

def find(*args, **kwargs):
    """ find matching objects. """
    return [x[1] for x in boht.run.store.find(*args, **kwargs)]

def kill(thrname):
    """ stop a thread. """
    for task in threading.enumerate():
        if thrname not in str(task):
            continue
        if "cancel" in dir(task):
            task.cancel()
        if "exit" in dir(task):
            task.exit()
        if "stop" in dir(task):
            task.stop()

def last(t):
    """ return last object of a type. """
    import boht.run
    v =  boht.run.store.last(t)
    if v:
        return v[-1]

def launch(func, *args, **kwargs):
    """ launch a task. """
    t = boht.tasks.Task()
    t.start()
    t.put(func, args, kwargs)
    return t

def load(self, path):
    """ load a object from disk and return it properly typed. """
    return boht.base.Object().load(path)

def start(name="boht", modules=[], version=__version__, wd=hd(".boht"), shell=False):
    """ application boot code. """
    boht.shell.termsave()
    global cfg
    try:
        boht.shell.parse_cli(name, version=version, wd=wd)
        cfg.shell = (not cfg.shell) or shell
        if cfg.owner:
            boht.run.users.oper(cfg.owner)
        if modules:
            cfg.modules += "," + ",".join(modules) 
        boht.shell.writepid()
        if cfg.background:
            logging.error("switching to daemon mode.")
            boht.shell.daemon()
        k = boht.run.kernel
        k.initialize(cfg.modules.split(","))
        k.start()
        k.tail()
    except KeyboardInterrupt:
        print("")
    except Exception as ex:
        logging.error(boht.trace.get_exception())
    boht.shell.reset()
    os._exit(0)

import boht.irc 
import boht.kernel
import boht.run
