""" timers, repeaters. """

## imports

import boht
import threading
import time

## classes

class Cfg(boht.base.Object):

    """ clock config. """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.latest = 0

class Timers(boht.base.Object):

    """ hold all running timers. """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._stopped = False
        self.cfg = Cfg()
        self.timers = boht.base.Object()

    def loop(self):
        """ poll for timers to trigger. """
        while not self._stopped:
            time.sleep(1.0)
            remove = []
            for t, event in self.timers.items():
                if time.time() > t:
                    self.cfg.latest = time.time()
                    self.cfg.save()
                    event.raw(event.txt)
                    remove.append(t)
            for r in remove:
                del self.timers[r]

    def start(self, type=None):
        """ start timers. """
        for e in boht.store.all("boht.clock.Timers"):
            if "done" in e and e.done:
                continue
            if "time" not in e:
                continue
            if time.time() < int(e.time):
                self.timers[e.time] = e
        return boht.launch(self.loop)

    def stop(self):
        """ stop timer loop. """
        self._stopped = True

class Timer(boht.base.Object):

    """ basic timer. """

    def __init__(self, sleep, func, *args, **kwargs):
        super().__init__()
        self._func = func
        self._name = kwargs.get("name", boht.utils.get_name(func))
        self.sleep = sleep
        self.args = args
        self.kwargs = kwargs
        self.state = boht.base.Object()

    def start(self):
        """ start the timer. """
        timer = threading.Timer(self.sleep, self.run, self.args, self.kwargs)
        timer.setName(self._name)
        timer.sleep = self.sleep
        timer.state = self.state
        timer.state.starttime = time.time()
        timer.state.latest = time.time()
        timer._func = self._func
        timer.start()
        return timer

    def run(self, *args, **kwargs):
        """ launch timer thread. """
        self.state.latest = time.time()
        boht.launch(self._func, *args, **kwargs)

    def exit(self):
        """ exit (stop) the timer. """
        self.cancel()

class Repeater(Timer):

    """ repeat a timer. """

    def run(self, *args, **kwargs):
        self._func(*args, **kwargs)
        return boht.launch(self.start)
