""" basic commands. """

## imports

import json
import os
import time
import threading

import boht

## commands

def ed(event):
    """
        edit the last saved object of a type.
        
        ed <type> <var=value>
        
        1) ed log txt="test"
        
    """
    if not event.selector:
        event.reply("ed <selector> <setter>")
        return
    o = boht.last(event.match)
    if not o:
        return
    if not event.setter:
        event.reply(o)
        return
    boht.edit(o, event.setter)
    fn = o.save() 
    event.reply("ok 1")

def fleet(event):
    """
        display list of running bots or the bot it self.

        fleet [nr]

        1) fleet
        2) fleet 1
        
    """
    try:
        nr = int(event.args[0])
    except (IndexError, ValueError):
        event.reply(str([boht.utils.get_type(b) for b in boht.run.kernel.bots]))
        return
    try:
        event.reply(str(boht.run.kernel.bots[nr]))
    except IndexError:
        pass

def find(event):
    """
        find objects.

        find <type> <selector>
        
        1) find log txt==test 

    """
    if len(event.args) < 1:
        event.reply("find <%s>" % "|".join(boht.static.classes.keys()))
        return
    elif len(event.args) == 1:
        o = boht.last(event.match)
        if o and not event.selector:
            for a in o.keys():
                if a not in event.selector:
                    event.selector[a] = None
                if a not in event.dkeys:
                    event.dkeys.append(a)
    stime = time.time()
    nr = -1
    for p, o in boht.run.store.find(event.match, event.selector, event.index, event.delta):
        txt = ""
        if "k" in event.options:
            event.reply("|".join(o.keys()))
            return
        if "d" in event.options:
            event.reply(str(o))
            continue
        full = False
        if "f" in event.options:
            full = True
        nr += 1
        if event.dkeys:
            txt = "%s %s" % (event.index or nr, boht.utils.format(o, event.dkeys, full))
        else:
            txt = "%s %s" % (event.index or nr, boht.utils.format(o, full=full))
        if "t" in event.options:
            txt += " " + boht.utils.days(p)
        if txt:
            event.reply(txt)

def load(event):
    """
        load a module.
        
        load <modulename>
        
        1) load boht.entry

    """
    if not boht.run.users.allowed(event.origin, "OPER"):
        event.reply("missing OPER permission.")
        return
    if not event.args:
        event.reply("|".join(sorted(boht.loader.table.keys())))
        return
    name = event.args[0]
    try:
        mod = boht.run.kernel.walk(name, True)
    except ModuleNotFoundError:
        event.reply("module %s not found." % name)
        return
    except:
        event.reply(boht.trace.get_exception())
    event.reply("%s loaded" % name)

def meet(event):
    """
        introduce a user to the bot
        
        meet <userhost|nick>
        
        1) meet botfather@shell.dds.nl
        2) meet botfather

    """
    if not event.args:
        event.reply("meet origin [permissions]")
        return
    try:
        origin, *perms = event.args[:]
    except ValueError:
        event.reply("|".join(sorted(boht.run.users.userhosts.keys())))
        return
    origin = getattr(boht.run.users.userhosts, origin, origin)
    u = boht.run.users.meet(origin, perms)
    event.reply("added %s" % u.user)

def rm(event):
    """
        remove an object (set it's _deleted flag)

        rm <type> <selector>

        1) rm log txt==test

    """
    if not event.args:
        event.reply("rm <selector>")
        return
    st = time.time()
    nr = -1
    for o in boht.find(event.match, event.selector, event.index, event.delta):
        nr += 1
        o._deleted = True
        o.save()
    event.reply("ok %s %s" % (nr+1, boht.utils.elapsed(time.time()-st)))

def show(event):
    """
        display runtime data. 
        
        show <cfg|cmds|help|mods|threads|types|uptime|version>

    """
    try:
        cmd, *args = event.args
    except ValueError:
        event.reply("cfg|cmds|help|mods|threads|types|uptime|version")
        return
    if cmd == "cfg":
        event.reply(boht.cfg)
    elif cmd == "cmds":
        event.reply("|".join(sorted(boht.static.names.keys())))
    elif cmd == "mods":
        event.reply("|".join(sorted(boht.static.modnames.values())))
    elif cmd == "help":
        res = ""
        for key, val in boht.static.aliases.items():
            res += "%s (%s) " % (key, val.split()[-1])
        if res:
            event.reply(res)
    elif cmd == "threads":
        psformat = "%-8s %-60s"
        result = []
        try:
            getnr = int(event.args[1])
        except (ValueError, IndexError):
            getnr = None
        for thr in sorted(threading.enumerate(), key=lambda x: x.getName()):
            d = vars(thr)
            o = boht.base.Object()
            o.update(d)
            if o.get("sleep", None):
                up = o.sleep - int(time.time() - o.state.latest)
            else:
                up = int(time.time() - boht.starttime)
            result.append((up, thr.getName(), o))
        nr = -1
        for up, thrname, o in sorted(result, key=lambda x: x[0]):
            nr += 1
            if getnr is not None and getnr == nr:
                event.reply(o)
                return
            res = "%s %s" % (nr, psformat % (boht.utils.elapsed(up), thrname[:30]))
            event.reply(res)
    elif cmd == "types":
        event.reply("|".join(boht.static.types))
    elif cmd == "version":
        event.reply("%s %s" % (boht.cfg.name.upper(), boht.cfg.version))
    elif cmd == "uptime":
        event.reply(boht.utils.elapsed(time.time() - boht.starttime))

def stop(event):
    """ stop a thread (task). """
    if not event.args:
        event.reply("stop <threadname>")
        return
    name = event.args[0]
    boht.kill(name)
    event.reply("kill %s" % name)

def undel(event):
    """
        undelete an object (set _deleted attribute to False).
        
        undel <type> <selector>
        
        1) undel log txt==test     
        
    """
    if not event.args:
        event.reply("undel <selector>")
        return
    st = time.time()
    nr = -1
    for o in boht.find(event.match, event.selector, event.index, event.delta, showdel=True):
        nr += 1
        o._deleted = False
        o.save()
    event.reply("ok %s %s" % (nr+1, boht.utils.elapsed(time.time()-st)))

def unload(event):
    """
        unload a module.
        
        unload <modulename>
        
        1) unload boht.entry    
        
    """
    if not event.args:
        event.reply("|".join(sorted(boht.loader.table.keys())))
        return
    if not boht.run.users.allowed(event.origin, "OPER"):
        event.reply("missing OPER permission.")
        return
    name = event.args[0]
    try:
        del boht.loader.table[name]
    except KeyError:
        event.reply("%s is not loaded." % name)        
        return
    event.reply("unload %s" % name)
