""" list of bots.  """

## imports

import boht

## classes

class Fleet(boht.base.Object):

    """ list of bots. """

    def __init__(self):
        super().__init__()
        self.bots = []

    def __iter__(self):
        """ iterate of fleet bots. """
        return iter(self.bots)

    def add(self, bot):
        """ add a bot to the fleet. """
        self.bots.append(bot)
        return self

    def announce(self, txt):
        """ announce txt on all bots. """
        for b in self.bots:
            b.announce(str(txt))

    def by_type(self, btype):
        """ return bots by type. """
        res = None
        for b in self.bots:
            if str(btype).lower() in str(type(b)).lower():
                res = b
        return res

    def get_bot(self, id):
        """ return bot matching corresponding id. """
        res = None
        for b in self.bots:
            if str(id) in repr(b):
                res = b
                break
        return res

    def get_firstbot(self):
        """ get first bot of fleet. """
        return self.bots[0]

    def match(self, m):
        """ match a bot by string. """
        res = None
        for b in self.bots:
            if m.lower() in repr(b):
                res = b
                break
        return res

    def remove(self, bot):
        """ remove bot from fleet. """
        self.bots.remove(bot)

    def save(self):
        """ call save on all bots. """
        for b in self.bots:
            b.save()

    def say(self, id, channel, txt):
        """ find matching bot and echo txt to the channel. """
        b = self.get_bot(id)
        if b:
            b.say(id, channel, txt)

    def stop(self):
        """ call stop on all bots. """
        for b in self.bots:
            b.stop()

    def wait(self):
        """ call wait on all bots. """
        for b in self.bots:
            b.wait()
