""" boot code and main handler.  """

## imports

import boht
import logging

## classes

class Kernel(boht.fleet.Fleet, boht.handler.Handler, boht.base.Store):

    """ basic boot code plus fleet and store functionality. """

    def __init__(self):
        super().__init__()
        self.cfg = boht.Cfg()

    def cmd(self, txt, options="", origin=""):
        """ exec a command. """
        e = boht.Event()
        e.txt = txt
        e.orig = repr(self)
        e.options = options
        e.origin = origin or "root@shell"
        self.dispatch(e)
        self.ready()
        return e

    def initialize(self, mods):
        """ initialize modules. """
        res = []
        thrs = []
        exclude = boht.cfg.exclude.split(",")
        for m in mods:
            if not m or m in exclude:
                continue
            logging.warn("init %s" % m)
            try:
                thrs = self.walk(m, True)
                for thr in thrs:
                    thr.join()
            except ModuleNotFoundError:
                logging.error("not found %s" % m)
            except Exception as ex:
                logging.error(boht.trace.get_exception())

    def start(self):
        """ start the kernel. """
        self.cfg.update(boht.cfg)
        if self.cfg.args:
            self.cfg.verbose = False
        super().start()

    def tail(self):
        """ wait for kernel to finish. """
        if self.cfg.args:
            self.cfg.verbose = False
            e = boht.cmd(" ".join(self.cfg.args))
            for txt in e._result:
                print(txt)
            return e
        if self.cfg.shell:
            c = boht.shell.Console()
            c.start()
            c.verbose = (not self.cfg.verbose) or True
            self.add(c)
            super().wait()
