""" module loader. """

## imports

import boht
import logging
import importlib
import inspect
import os
import pkgutil
import time
import types

## vars

#:
table = {}

## classes

class Loader(boht.base.Object):

    """ loads modules into the table. """

    def cached(self, name):
        """ return cached module if available. """
        global table
        if name not in table:
            table[name] = self.direct(name) 
        return table[name]

    def direct(self, name, package=None):
        """ direct load into table. """
        logging.debug("load %s" % name)
        try:
            p, m = name.split(" .", 1) 
            return importlib.import_module(m, p)
        except ValueError:
            return importlib.import_module(name)

    def load_mod(self, name, mod=None):
        """ load a module, scan if for commands. """
        mod = mod or self.cached(name)
        for key, func, modname in self.scan_commands(mod):
            boht.static.names[key] = modname
        return mod

    def scan_commands(self, mod):
        """ scan a module for commands (contains a event parameter). """
        for key in dir(mod):
            if key.startswith("_"):
                continue
            o = getattr(mod, key, None)
            if o and isinstance(o, types.FunctionType):
                if "event" in o.__code__.co_varnames:
                    yield (key, o, mod.__name__)
            elif o and isinstance(o, boht.base.Object):
                t = boht.utils.get_type(o)
                if t not in boht.static.types:
                    boht.static.types.append(t)
                cls = t.split(".")[-1]
                if cls not in boht.static.classes:
                    boht.static.classes[cls.lower()] = t

    def unload(self, modname):
        """ unload a module from table. """
        global handlers, table
        if modname in table:
            mod = table[modname]
        for key, func, name in self.scan(mod):
            if name == modname:
                try:
                    del handlers[key]
                    del boht.static.names[key]
                except KeyError:
                    pass
