""" threads. """

## imports

import boht
import logging
import queue
import threading
import time

## classes

class Task(threading.Thread):

    """ bot's own threads. """

    def __init__(self, *args, **kwargs):
        super().__init__(None, self.run, "", [], kwargs, daemon=True)
        self._queue = queue.Queue()
        self._func = None
        self._ready = threading.Event()
        self._result = []
        self._stopped = False

    def __iter__(self):
        """ iterate over thread's namespace. """
        return self

    def __next__(self):
        for k in dir(self):
            yield k

    def parse(self, txt):
        """ parse method stub. """
        pass

    def put(self, func, args, kwargs):
        """ send a function with args/kwargs for processing. """
        self._queue.put((func, args, kwargs))

    def run(self):
        """ basic code to run in a task (fetch from queue, exec, return result). """
        (func, args, kwargs) = self._queue.get()
        try:
            e = args[0]
            txt = e.txt
        except (IndexError, AttributeError):
            txt = boht.utils.get_name(func)
        self.setName(txt)
        try:
            res = func(*args, **kwargs)
            self._result.append(res)
        except Exception as ex:
            logging.error(boht.trace.get_exception())
        self._ready.set()
        return self._result

    def join(self, timeout=None):
        """ join a thread, return the result. """
        super().join(timeout)
        return self._result

    def stop(self):
        """ stop the thread. """
        self._stopped = True
        self._queue.put(None)

    def wait(self):
        """ wait for thread to finish. """
        self._ready.wait()
        return self._result
