""" manage users. """

## imports

import boht
import logging

## exceptions

class ENOUSER(Exception):
    """ user was not found. """
    pass

## classes

class User(boht.base.Object):

    """ contains userhost and permissions of a user. """

    def __init__(self):
        super().__init__()
        self.user = ""
        self.perms = []

class Users(boht.base.Store):

    """ handles multiple users. """

    cache = boht.base.Object()
    userhosts = boht.base.Object()

    def allowed(self, origin, perm):
        """ see if user if allowed to execute a command with permission perm. """
        perm = perm.upper()
        user = self.get_user(origin)
        if user:
            if perm in user.perms:
                logging.warn("allow %s %s" % (origin, ",".join(user.perms)))
                return True
        logging.error("denied %s %s" % (origin, perm))
        return False

    def delete(self, origin, perm):
        """ remove a user. """
        for user in self.get_users(origin):
            try:
                user.perms.remove(perm)
                user.save()
                return True
            except ValueError:
                pass

    def get_users(self, origin=""):
        """ return users matching origin. """
        s = {"user": origin}
        v = self.all("boht.users.User", s)
        if v:
            return v[-1][-1]

    def get_user(self, origin, type=None):
        """ return a single user. """
        if origin in dir(Users.cache):
            return getattr(Users.cache, origin, None)
        s = {"user": origin}
        res = self.find("boht.users.User", s)
        if res:
            p, u = res[-1]
            Users.cache.set(origin, u)
            return u

    def meet(self, origin, perms=[]):
        """ add a user. """
        user = self.get_user(origin)
        if not user:
            user = User()
        user.user = origin
        user.perms = perms + ["USER", ]
        if perms:
            user.perms.extend(perms)
        user.save(timed=True)
        return user

    def oper(self, origin):
        """ a temp. oper account to the cache. """
        user = User()
        user.user = origin
        user.perms = ["OPER", "USER"]
        Users.cache.set(origin, user)
        return user

    def perm(self, origin, permission):
        """ add permission to user object. """
        user = self.get_user(origin)
        if not user:
            raise ENOUSER(origin)
        if permission.upper() not in user.perms:
            user.perms.append(permission.upper())
            user.save()
        return user
